
const char *
wdns_rrclass_to_str(uint16_t rrclass)
{
	switch (rrclass) {
	case WDNS_CLASS_IN: return ("IN");
	case WDNS_CLASS_CH: return ("CH");
	case WDNS_CLASS_HS: return ("HS");
	case WDNS_CLASS_NONE: return ("NONE");
	case WDNS_CLASS_ANY: return ("ANY");
	}

	return (NULL);
}

static struct u16str {
    uint16_t u16;
    const char *str;
} rrclasses[] = {
	{ WDNS_CLASS_ANY, "ANY" },
	{ WDNS_CLASS_CH, "CH" },
	{ WDNS_CLASS_HS, "HS" },
	{ WDNS_CLASS_IN, "IN" },
	{ WDNS_CLASS_NONE, "NONE" },
};

#define num_rrclasses (sizeof(rrclasses) / sizeof(struct u16str))

static int
cmp_u16str(const void *a, const void *b) {
    struct u16str *u1 = (struct u16str *) a;
    struct u16str *u2 = (struct u16str *) b;
    return (strcasecmp(u1->str, u2->str));
}

static bool
convert_generic_rrclass(const char *s, long int *val) {
    char *endptr = NULL;

    if (strlen(s) <= 5)
        return (false);
    if (strncasecmp(s, "CLASS", 5) != 0)
        return (false);

    s += 5; /* skip leading "CLASS" */

    *val = strtol(s, &endptr, 10);
    if (endptr != NULL && *endptr != '\0')
        return (false);
    if (*val < 0 || *val > 65535)
        return (false);

    return (true);
}

uint16_t
wdns_str_to_rrclass(const char *str) {
    struct u16str key, *res;
    key.str = str;
    res = bsearch(&key, rrclasses, num_rrclasses, sizeof(struct u16str), cmp_u16str);
    if (res != NULL) {
        return (res->u16);
    } else {
        long int val = 0;
        if (convert_generic_rrclass(str, &val))
            return (val);
    }
    return (0);
}
